import json

import requests
import warnings
import pandas as pd
from persiantools import characters
from algotik_tse.settings import Settings
from algotik_tse.core.search import search_stock
warnings.simplefilter(action='ignore', category=FutureWarning)

settings = Settings()


def stockdetail(stock):
    """
    Get all symbol detail
    :param stock: name of stock
                  Default value is 'شتران'.

    :return: pandas dataframe
    """
    web_id = search_stock(search_txt=stock)
    if len(web_id) != 0:
        if web_id[-5:] == "index":
            print("This is an index, no information found!")
            return None
        else:
            detail = requests.get(settings.url_detail.format(web_id), headers=settings.headers)
            if detail.status_code == 200:
                df = pd.read_html(detail.content, encoding='utf8')[0]
                df.rename(columns={0: 'key', 1: 'value'}, inplace=True)
                df.set_index('key', drop=True, inplace=True)
                df.loc['id'] = str(web_id)
                return df
            else:
                print("Connection Error!!!")
                return None
    else:
        print("Stock Not Found, Please try again ...")
        return None


def __flatten_dict(dd, separator='_', prefix=''):
    return { prefix + separator + k if prefix else k : v
             for kk, vv in dd.items()
             for k, v in __flatten_dict(vv, separator, kk).items()
             } if isinstance(dd, dict) else { prefix : dd }


def stock_information(stock):
    """
    Get all stock information
    :param stock: name of stock
                  Default value is 'شتران'.

    :return: pandas dataframe
    """
    web_id = search_stock(search_txt=stock)
    if len(web_id) != 0:
        if web_id[-5:] == "index":
            print("This is an index, no information found!")
            return None
        else:
            detail = requests.get(settings.url_instrument_information.format(web_id), headers=settings.headers)
            if detail.status_code == 200:
                raw = __flatten_dict(detail.json()['instrumentInfo'])
                df = pd.DataFrame([raw]).T
                df.reset_index(inplace=True)
                df.rename(columns={'index': 'key', 0: 'value'}, inplace=True)
                df.set_index('key', inplace=True)
                return df
            else:
                print("Connection Error!!!")
                return None
    else:
        print("Stock Not Found, Please try again ...")
        return None


def stock_statistics(stock):
    """
        Get all stock statistics
        :param stock: name of stock
                      Default value is 'شتران'.

        :return: pandas dataframe
        """
    web_id = search_stock(search_txt=stock)
    if len(web_id) != 0:
        if web_id[-5:] == "index":
            print("This is an index, no statistics found!")
            return None
        else:
            detail = requests.get(settings.url_instrument_statistics.format(web_id), headers=settings.headers)
            if detail.status_code == 200:
                js = detail.json()['instrumentStatistic']
                statistics_dict = {
                    characters.ar_to_fa(x['dataTypeDesc']): float(x['dataValue']) if '.' in x['dataValue'] else int(
                        x['dataValue']) for x in js}
                raw = __flatten_dict(statistics_dict)
                df = pd.DataFrame([raw]).T
                df.reset_index(inplace=True)
                df.rename(columns={'index': 'key', 0: 'value'}, inplace=True)
                df.set_index('key', inplace=True)
                return df
            else:
                print("Connection Error!!!")
                return None
    else:
        print("Stock Not Found, Please try again ...")
        return None



import requests
import pandas as pd
from algotik_tse.core.search import search_stock
from algotik_tse.settings import Settings
from algotik_tse.core.helper import date_fix

settings = Settings()


def shareholders(stock="", date=None, shh_id=False):
    """
    Get all shareholders of an instrument (company)
    :param stock: name of stock
                  Default value is 'شتران'.
    :param date: date for get shareholders in specific date, if None the function return last shareholders
                  Default value is None
    :param shh_id: if is True, shareholder id will be shown in df and if is False, shareholder not show
                  Default value is False

    :return: pandas dataframe
    """
    web_id = search_stock(search_txt=stock)
    if len(web_id) != 0:
        try:
            if date is None:
                share_holder_id_name = 'shareHolderShareID'
                _shareholders_base_url = settings.url_last_share_holders
                response = requests.get(url=_shareholders_base_url.format(web_id), headers=settings.headers)
            else:
                share_holder_id_name = 'shareHolderID'
                new_start, new_end = date_fix(start=date)
                new_start = new_start.replace("-", "")
                _shareholders_base_url = settings.url_share_holders_history
                url = _shareholders_base_url.format(web_id, new_start)
                response = requests.get(url=url, headers=settings.headers)
            if response.status_code == 200:
                share_holders = response.json()['shareHolder'] if date is None else response.json()['shareShareholder']
                share_holders_df = pd.DataFrame(share_holders)
                share_holders_columns = ['shareHolderName', 'numberOfShares', 'perOfShares', 'change', 'changeAmount',
                                         'dEven']
                if shh_id:
                    share_holders_columns.append(share_holder_id_name)
                share_holders_df = share_holders_df.loc[:, share_holders_columns]
                share_holders_df.rename(columns={'shareHolderName': 'share_holder_name',
                                                 'numberOfShares': 'number_of_shares',
                                                 'perOfShares': 'percentage_of_shares', 'change': 'change_state',
                                                 'changeAmount': 'change_amount', 'dEven': 'date'}, inplace=True)
                if shh_id:
                    share_holders_df.rename(columns={share_holder_id_name: 'share_holder_id'}, inplace=True)
                if date is not None:
                    share_holders_df['first_row'] = share_holders_df.groupby('share_holder_name')['number_of_shares'].transform('first')
                    share_holders_df['last_row'] = share_holders_df.groupby('share_holder_name')[
                        'number_of_shares'].transform('last')
                    share_holders_df['change_amount'] = share_holders_df['first_row'] - share_holders_df['last_row']
                    share_holders_df = share_holders_df.drop(['first_row', 'last_row'], axis=1)
                    share_holders_df = share_holders_df.loc[share_holders_df['date'] == share_holders_df['date'].max(),
                                       :]
                else:
                    share_holders_df['date'] = settings.today.replace('-', '')
                return share_holders_df
        except:
            print("Connection Error!!!")
            return None
    else:
        print("Stock Not Found, Please try again ...")
        return None
